%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% DemoGreenHighContrast
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Overview:
%   Green's function in high contrast media
%
% References:
%   Section 9.2
%   Mathematical and Computational Methods in Photonics and Phononics
%
% Authors:
%   Habib Ammari, Brian Fitzpatrick, Hyeonbae Kang,
%   Matias Ruiz, Sanghyeon Yu, Hai Zhang
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
clc, clear all, close all
% --------------------------------------------------
% Create a disk.
% --------------------------------------------------
R = 2; % radius of a disk
nbPoints = 300; %number of points on the boundary
theta = 2*pi*(0:nbPoints-1)/nbPoints;
D=ellipse(R,R,0,0,nbPoints); 

% --------------------------------------------------
% Set frequency and material parameters.
% --------------------------------------------------
omega=1;
eps_m = 1; eps_c = 1;
mu_m = 1;

k_m=omega*sqrt(eps_m*mu_m); 

% --------------------------------------------------
% Find the distribution of the resonant values for k_c
% (approximately).
% --------------------------------------------------
NN=500;
kc_start=0;
kc_end=10;
kc_values = kc_start: (kc_end-kc_start)/NN :kc_end;
det_values = zeros(1,NN+1);
for i=1:NN+1
    k_c = kc_values(i);
    mu_c = k_c^2/(omega^2*eps_c);
    
    Jc = besselj(0,k_c*R);
    Jc_deri = -besselj(1,k_c*R);
    Hm = besselh(0,1,k_m*R);
    Hm_deri = -besselh(1,1,k_m*R);

    det_values(i) = k_c/mu_c*Jc_deri*Hm - k_m/mu_m*Jc*Hm_deri;
end

figure(1)
plot(kc_values,abs(det_values), '-', 'color','blue');

% --------------------------------------------------
% Set the parameters for high contrast media.
% --------------------------------------------------
k_c=5.9;
mu_c = k_c^2/(omega^2*eps_c);

% --------------------------------------------------
% Solving the integral equation.
% --------------------------------------------------
A11 = SingleL_Helm(k_c,D,D);
A12 = -SingleL_Helm(k_m,D,D);
A21 = (1/mu_c)*( (-1/2)*eye(D.nbPoints) + KDstar_Helm(k_c,D) );
A22 = (-1/mu_m)*( (+1/2)*eye(D.nbPoints) + KDstar_Helm(k_m,D) );
MatA = [A11, A12 ; A21, A22];

 
b1 = (-1)*(-1i/4*besselh(0,1,k_c*R))*ones(1,nbPoints);
b2 = (-1/mu_c)*(-1i/4)*k_c*( -besselh(1,1,k_c*R)*ones(1,nbPoints) ) ;

vecB = [b1.';b2.'];


% get the density functions \phi and \psi
sol = MatA\vecB; 
phi = sol(1:nbPoints);
psi = sol(nbPoints+1:2*nbPoints);


%% Compute the numerical solution (scattered wave) on the circle |x|=2 
green = @(k,x,y) -1i/4*besselh(0,1,k*sqrt(x.^2+y.^2));
NN=100;
Omega=zeros(2,2*NN-1);
G0=zeros(2*NN-1,1);
Gc=zeros(2*NN-1,1);
for j=1:2*NN-1
Omega(1,j) = -R+R/NN*j;
Gc(j) = green(k_c,-R+R/NN*j,0);
G0(j) = green(k_m,-R+R/NN*j,0);
end



us_num = Gc+SingleL_Helm2(k_c,Omega,D)*phi;



%% Compute exact analytical solution (scattered wave) on the circle |x|=2


Jc = besselj(0,k_c*R);
Jc_deri = -besselj(1,k_c*R);
Hc = besselh(0,1,k_c*R);
Hc_deri = -besselh(1,1,k_c*R);
Hm = besselh(0,1,k_m*R);
Hm_deri = -besselh(1,1,k_m*R);

det = k_c/mu_c*Jc_deri*Hm - k_m/mu_m*Jc*Hm_deri;
a=(1i/4)*1/det*(k_c/mu_c*Hm*Hc_deri - k_m/mu_m*Hc*Hm_deri);
b=(1i/4)*1/det*(k_c/mu_c*Jc*Hc_deri - k_c/mu_c*Jc_deri*Hc);




us_theo = Gc+exact_sol(k_c,k_m,mu_m,mu_c,R,NN);

us_ref = G0;
xvalue = zeros(2*NN-1,1);
for j=1:2*NN-1
    xvalue(j) = -R+j/NN*R;
end
%% Plot exact sol. and numerical sol. for comparison 
figure(2)
hold on

plot(xvalue, -imag(us_theo), 'o', 'color','blue');
plot(xvalue, -imag(us_num), 'r-','linewidth',1.5);
plot(xvalue, -imag(us_ref), '--','color','black');
legend('-Im(G_{theo})','-Im(G_{num})','-Im(G_{0})','Location','southeast')
axis([-R,R,-1,1.5]);
